<?php 
include_once 'Task.class.php';
include_once 'ECATP.class.php';

/**
 * @brief PoolThread Classe gérant le pool de worker
 * @detail Par défaut le mode est ROUND-ROBIN mais on peut le spécifier d'autre mode de fonctionnement 
 * @author saloc
 *
 */
class PoolThread extends Pool
{
	/** @brief constante modifiant le comportement par défaut du pool*/
	const AUTOSIZE=1;
	
	/** @brief Nom du pool*/
	private $name;
	
	/** @brief mode de fonctionnement du pool (autosize ou pas)*/
	private $mode;
	
	private $collect_function;
	/**
	 * @brief constructeur par défaut
	 * @param integer $size taille du pool de Worker
	 * @param integer $modes (default:0) mode de gestion du pool 
	 * 							si $mode = 0 alors c'est du round robin simple.
	 * 							si mode = PoolThread::AUTOSIZE le pooler ajoutera automatiquement un thread/worker si tout les autres sont occupés.
	 */
	public function __construct ($poolname,$size, $modes=0,$fctcollect=null){
		$this->name = $poolname;
		$this->mode = $modes;
		$this->collect_function = $fctcollect;
		
		parent::__construct($size,Worker::class);
	}
	
	public function submitTask($object){
	if($this->mode==PoolThread::AUTOSIZE){ //Si on se retrouve avec un travail en attente, alors on augmente le nbre de threads de 1.
			$resize = false;
			foreach($this->workers as $worker){	
				if($worker->getStacked()>0) {   //1 thread a 1 travail en attente alors on crée un nouveau thrad
					$resize=true;
					break;
				}
			}
			if($resize) $this->resize($this->size+1);
		}
		$this->submit($object);
	}
	
	public function collectTask(){
		$this->collect($this->collect_function);
	}
	
	/**
	 * @brief retourne la taille du Pool de Thread
	 * @return integer la taille du pool
	 */
	public function getPoolSize()
	{
		return count($this->workers);
	}
}
?>