<?php

/**
 * @brief Classe définnisant un objet de type Task qui est utilisé par ECATP pour l'execution des threads.
 */

class Task extends Threaded implements Collectable
{
	/** CONSTANTE
	 *  liste des valeur que peut prendre $status et indique l'état de l'exécution du plugin
	 */
	
	/** @brief  l'exécution ne sait pas encore passé */
	const TASK_PENDING_EXEC = -1;
	/** @brief  l'exécution ne sait pas encore passé */
	const TASK_FINISH_EXEC = -2;
	/** @brief  l'exécution c'est bien déroulé */
	const TASK_EXEC_SUCCESS = 0;   
	/** @brief  l'exécutable est introuvable sur la SD mais définis dans le tableau*/
	const TASK_EXEC_NO_FOUND = 1;    
	/** @brief  le plugin n'est pas défini dans le tableau d'exécution*/
	const TASK_PLUGIN_NOT_DEFINED=2;
	/** @brief  la GESA est injoignable*/
	const TASK_GESA_UNREACHABLE = 3;
	/** @brief  le code de retour de plugin n'est pas supporté*/
	const TASK_EXEC_BAD_CODE_RETURN = 4;
	/** @brief  l'exécution du plugin est bloqué ou prends trop de temps*/
	const TASK_EXEC_TIMEOUT = 5;
	/** @brief  une erreur d'execution non référencée c'est produite */
	const TASK_EXEC_FAILED = 255;    
	
	/** @brief Identifiant de la tache donnée par le serveur */
	public $taskID;
	/** @brief nom du plugin (ex:check_snmp_load.pl) */
	public $name;
	/** @brief Paramètre d'execution du plugin  */
	public $parameter;
	/** @brief la sortie standard du plugin après execution  */
	public $output;
	/** @brief le code de retour du plugin après execution */
	public $return;
	/** @brief le status de la tache */
	public $status;
	/** @brief une priorité */
	public $priority;	
	/** @brief la commande à exécuter */
	public $command;
	/** @brief le moment de l'envoit de la demande d'exécution*/ 
	public $time;

	/**
	  @brief Constructeur par défaut 
	  @param integer $task l'identifiant de la tache.
	  @param string $name le nom du plugin
	  @param string $param les paramètres d'executions  
	  @param integer $priorite un priorité de 1 à 7 (default:5)
	  @return void
	 */
	public function __construct($taskid,$name,$parameter,$priorite = 5)
	{
		$this->taskID = $taskid;
		$this->name = $name;
		$this->parameter = $parameter;
		$this->priority = $priorite;
		$this->status = Task::TASK_PENDING_EXEC;
	}
	/**
	 * @brief Destructeur
	 * @detail Destructer, libère les ressource de task et de complete
	 * @return void
	 */
	public function __destruct ()
	{

	}
	public function destroy()
	{
		$this->taskID = null;
		$this->name = null;
		$this->parameter = null;
		$this->output = null;
		$this->return = null;
		$this->status = null; 
		$this->priority = null; 
		$this->command = null;
	}
	/**
	 @brief Retourne une description sous forme de chaine
	 @detail affiche l'objet sous forme de texte lorsqu'on l'appelle via echo
	 @return string Une chaine de caractère représentant l'objet
	 */
	public function __toString()
	{
		return print_r($this,TRUE);
	}
	/**
	  @brief Set le TaskID de la tâche
	  @param integer $task l'identifiant de la tache.
	  @return True si l'affectation est reussie sinon FALSE si ke paramètre n'est pas un integer.
	 */
	public function setTaskId( $task)
	{
		if(is_int($task))
		{
			$this->taskID = $task;
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
	/**
	  @brief Set le nom de la tâche
	  @param string $name le nom du plugin
	  @return void
	 */
	public function setName( $name)
	{
		$this->name = $name;
	}	
	/**
	  @brief Set les paramètres de la tâche
	  @param string $param les paramètres d'executions
	  @return void
	 */
	public function setParameter( $param)
	{
		$this->parameter = $param;
	}	
	/**
	  @brief Set la valeur de retour de l'execution 
	  @param string $return la valeur de retour du plugin compris entre 0 et 4
	  @return True si l'affectation est reussie sinon FALSE si le paramètre n'est pas compris entre 0 et 4
	 */
	public function setReturn( $return)
	{
		if(is_int($return) && ($return >=0 || $return <=4))
		{
			$this->return = $return;
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
	/**
	  @brief Set la sortie standart de la tâche
	  @param string $out le nom du plugin
	  @return void
	 */
	public function setOutput( $out)
	{
		$this->output = $out;
	}
	/**
	  @brief Set le status d'execution du plugin
	  @param string $status le status du plugin (voir définition des constantes "TASK_*")
	  @return void
	 */
	public function setExecutionStatus( $status)
	{
		$this->status = $status;
	}	
	/**
	 * @brief Set la priorité de la tâche
	 * @param integer $pri
	 * @return void
	 */
	public function setPriority( $pri)
	{
		$this->priority = $pri;
	}
	/**
	 * @brief Set la commande a exécuter
	 * @param integer $com le path de la commande a exécuter
	 * @return void
	 */
	public function setCommand( $com)
	{
		$this->command = $com;
	}
	/**
	 * @brief permet d'indiquer quand a été envoyer la demande d'exécution
	 * @param timestamp $time
	 */
	public function setSendExecTIme($time)
	{
		$this->time = $time;
	}
	/**
	  @brief Retourne le TaskId
	  @return integer
	 */
	public function getTaskID()
	{
		return $this->taskID;
	}	
	/**
	  @brief Retourne le nom du plugin
	  @return string
	 */
	public function getName()
	{
		return $this->name;
	}
	/**
	  @brief Retourne les paramètres du plugin
	  @return string
	 */
	public function getParameter()
	{
		return $this->parameter;	
	}
	/**
	  @brief Retourne le code de retour du plugin executé 
	  @detail Retourne le code de retour du plugin executé 
	  			0=OK
	  			1=WARNING
	  			2=CRITIQUE
	  			3=UNKNOWN
	  			4=AUTRE
	  @return integer
	 */
	public function getReturnCode()
	{
		return $this->return;
	}
	/**
	  @brief Retourne la sortie standard du plugin (stdout)
	  @return string
	 */
	public function getOutput()
	{
		return $this->output;
	}
	/**
	  @brief Retourne le code de l'état d'execution
	  @detail Retourne le code de l'état d'execution (voir définition des constantes "TASK_*")
	         voir la méthode ExecutionMessage pour avoir le message de l'erreur
	  @return integer
	 */
	public function getExecutionStatus()
	{
		return $this->status;
	}
	/**
	 * brief Retourne la priorité de la tâche
	 * @return integer
	 */
	public function getPriority()
	{
		return $this->priority;
	}
	/**
	 * brief Retourne la commande a exécuter
	 * @return integer
	 */
	public function getCommand()
	{
		return $this->command;
	}
	/**
	 * @brief retourne le timestamp de quand la demandee d'éxécution a été envoyée
	 * @retval timestamp $time
	 */
	public function getSendExecTime()
	{
		return $this->time;
	}
	/**
	  @brief Retourne le message associé au code de l'ExecutionStatus
	  @return string
	 */	
	public function ExecutionMessage()
	{
		$message="";
		switch ($this->status)
		{
			case $this::TASK_EXEC_SUCCESS: $message="Execution success"; break;
			case $this::TASK_EXEC_NO_FOUND: $message="Executable not found"; break;
			case $this::TASK_PLUGIN_NOT_DEFINED: $message="Plugin not defined in the configuration file"; break;
			case $this::TASK_GESA_UNREACHABLE: $message="The GESA is not connected to the WebSocket server"; break;
			case $this::TASK_EXEC_FAILED: $message=$this->executionmessage; break;
			case $this::TASK_EXEC_BAD_CODE_RETURN: $message="Bad code return (\$return >= 0 && \$return <=0)"; break;
			case $this::TASK_EXEC_TIMEOUT: $message="GESA delay for response is expired."; break;
			default: $message='Status not defined'; break;
		}
		return $message;
	}
	/**
	  @brief Vérifie que l'execution c'est bien passé
	  @return boolean Retourne TRUE si l'execution c'est bien déroulée et false si non
	 */	
	public function exec_isSuccess()
	{
		if($this->status == $this::TASK_EXEC_SUCCESS)
		{
			return TRUE;
		}
		else 
		{
			return FALSE;
		}
	}
	/**
	 @brief execute le programme
 	 @param string $out le nom du plugin	 
	 @detail execute le programme passé via le path en y ajoutant les parametre contenu dans l'objet
	 @return boolean Retourne TRUE si l'execution c'est bien déroulée et false si non
	 */
	public function execute()
	{	
		echo "\t\t$this->taskID : EXEC TASK: $path $this->parameter\n";
		//sécurise contre des injections au niveau des paramètres.
		$path = $this->command.' '.escapeshellcmd($this->parameter);
		//	echo "\t\t$this->taskID : EXEC TASK: $path\n";
			
			exec($path,$stdout,$return);
			//on test pour voir s'il n'y a pas eu d'erreur d'execution
			if($return==127)
			{
				//erreur 127 = executable introuvable
				$this->setExecutionStatus(Task::TASK_EXEC_NO_FOUND);
			}
			else
			{
				//execution reussie du plugin
				$this->setExecutionStatus(Task::TASK_EXEC_SUCCESS);
			}
			//on regarde si le code de retour est bien compris entre 0 et 4
			if($this->setReturn($return)===FALSE )
			{
				$this->setExecutionStatus(Task::TASK_EXEC_BAD_CODE_RETURN);
			}
			//Si l'execution est un succès et que tout c'est bien passé
			$output="";
			foreach ($stdout as $line)
			{
				$output .= $line;
			}
			echo "RETURN: $output ==> $return\n";
			$this->setOutput($output);
	}
	/**
	 * @brief méthode d'exécution
	 * @detail la méthode run parmet d'exécuter le plugin demandé et de retournée l'état de la tâche à ecapt
	 */
	public function run ( )
	{
		try
		{
			if(!empty($this->command))
			{
				//creation du path
				$path = $this->command.' '.escapeshellcmd($this->parameter);
				
				//descripteur pour le stdout et le stderr
				$descriptorspec = array(
						1 => array("pipe", "w"),  // stdout est un pipe où le processus va écrire
						2 => array("pipe", "w") // stderr est un fichier
				);
				
				//on ouvre la ressource
				$process = proc_open($path, $descriptorspec, $pipes); 
				if (is_resource($process)) 
				{
					exec($path,$stdout,$return);
					//on test pour voir s'il n'y a pas eu d'erreur d'execution
					if($return==127)
					{
						//erreur 127 = executable introuvable
						$this->setExecutionStatus(Task::TASK_EXEC_NO_FOUND);
					}
					//on regarde si le code de retour est bien compris entre 0 et 4
					elseif($this->setReturn($return)===FALSE )
					{
						$this->setExecutionStatus(Task::TASK_EXEC_BAD_CODE_RETURN);
					}
					else
					{
						//execution reussie du plugin
						$this->setExecutionStatus(Task::TASK_EXEC_SUCCESS);
						//Si l'execution est un succès et que tout c'est bien passé
						$output="";
						foreach ($stdout as $line)
						{
							$output .= $line."\n";
						}
						$this->setOutput($output);
						
					}
				}
				else
				{
						$this->setExecutionStatus(Task::TASK_EXEC_FAILED);
				}
			}
			else
			{
				$this->setExecutionStatus(Task::TASK_PLUGIN_NOT_DEFINED);
			}
			//si l'execution c'est bien passé tout es OK
			if(!$this->exec_isSuccess())
			{
				$output = $this->ExecutionMessage();
				$this->setOutput($output);
			}
			//echo "\t\t$this->taskID END : $this->output ==> $this->return ==> $this->status\n";
		}
		catch(Exception $e)
		{
			echo $e->getMessage()."\n";
		}
	}
	/**
	 * @brief fonction statique qui transforme un objet TaskNotThreaded en objet Task
	 * @param TaskNotThreaded $task
	 * @retval Task un objet Task
	 */
	static function constructByTaskNotThreaded($task)
	{
		$insTask =  new Task($task->taskID, $task->name, $task->parameter);
		$insTask->setOutput($task->output);
		$insTask->setReturn($task->return);
		$insTask->setExecutionStatus( $task->status);
		$insTask->setPriority($task->priority);
		$insTask->setCommand($task->command);
		$insTask->setSendExecTIme($task->time);	
		return 	$insTask;	
	}
}
/**
 * @brief classe Task mais n'étant pas étendue par la classe threaded (problème d'immutable)
 * @author saloc
 *
 */
class TaskNotThreaded 
{
	/** @brief Identifiant de la tache donnée par le serveur */
	public $taskID;
	/** @brief nom du plugin (ex:check_snmp_load.pl) */
	public $name;
	/** @brief Paramètre d'execution du plugin  */
	public $parameter;
	/** @brief la sortie standard du plugin après execution  */
	public $output;
	/** @brief le code de retour du plugin après execution */
	public $return;
	/** @brief le status de la tache */
	public $status;
	/** @brief une priorité */
	public $priority;	
	/** @brief la commande à exécuter */
	public $command;
	/** @brief le moment de l'envoit de la demande d'exécution*/ 
	public $time;
	public function __construct(Task $task = null)
	{
		if(isset($task))
		{
			$this->taskID = $task->getTaskID();
			$this->name =$task->getName();
			$this->parameter =$task->getParameter();
			$this->output = $task->getOutput();
			$this->return = $task->getReturnCode();
			$this->status = $task->getExecutionStatus();
			$this->priority = $task->getPriority();
			$this->command = $task->getCommand();
			$this->time = $task->getSendExecTime();
		}
	}	
} 
?>