<?php
include_once 'ECATP.class.php';

/**
 * @brief Classe définissant un objet de type ThAlive qui est un thread chargé de maintenir la communication de la Web socket 
 */

class ThAlive extends Thread
{
	/** @brief un objet ECATP protocol*/
	private $ecatp ;
	/** @brief booleen de fin de thread*/
	private $end=0;
	/** @brief un timer en seconde qui spécifié l'interval d'envoit des messages PING-PONG*/
	private $timer;
	private $debug;
	/**
	 @brief Constructeur par défaut
	 @detail Contruit un objet ThAlive qui est un thread qui va de manière périodique envoyé un ALIVE via le protocole ECATP.
	 @param $ecatp un objet ECAPT
	 @param $timer (default:50) interval de temps entre les envoits des PING-PONG (Alive)
	 @param $vdebug (default:0) mode debug de l'objet
	 @return void rien nada c'est un constructeur :D
	 */
	public function __construct (ECATP &$ecatp,$timer=10,$vdebug = 0)
	{
		$this->ecatp = $ecatp;
		$this->timer=$timer;
		$this->debug = $vdebug;
	}
	/**
	 @brief Prévient le thread de se finir proprement.
	 @detail Set une variable de fin pour le thread thAlive
	 @return void 
	 */
	public function Terminated()
	{
		$this->end=1;
	}
	/**
	 @brief code executé par le thread
	 @detail envoit périodiquement 
	 @return void
	 */	
	public function run ( )
	{
	//	global $ECATP;
		$this->Trace("START THREAD");
		while ($this->end==0)
		{
			$this->Trace("SLEEP:".$this->timer );
			sleep($this->timer);
			try 
			{
				$this->Trace("SEND ALIVE:");
				$this->ecatp->sendAlive();
				gc_collect_cycles();
			}
			catch (SocketException $e)
			{
				$this->Trace('EXECEPTION ('.$e->getCode().'): '.$e->getMessage());
				$end =1;
			}
		}
		$this->Trace("END THREAD");
		exit(0);
		
	}
	/**
	 * @brief fonction de trace 
	 * @detail fonction utilisée lorsque l'objet est en mode debug.
	 * @param string $string la chaine à loggé
	 */
	private function Trace($string)
	{
		if($this->debug !=0)
		{
			if(is_array($string))
				echo date(DATE_RFC2822).'#ThAlive#'.print_r($string,true);
			else
				echo date(DATE_RFC2822)."#ThAlive#$string\n";
		}
	}
}
